/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QMainWindow>
#include <QMessageBox>
#include <QLabel>
#include <QSpinBox>
#include <QFileDialog>
#include <QDir>
#include <QTimer>
#include <QCloseEvent>
#include <QSettings>

#include "main_widget.h"
#include "../logic/defines.h"

MainWidget *gui_h;	/* global gui handle */

MainWidget::MainWidget(QWidget *parent)
	: QMainWindow(parent)
{
	gui_h = this;
	setupUi(this);
	spth = new SpellathonCls();
	settings = new QSettings("Girish", "Spellathon");

	statWordCnt = new QLabel(this);
	statMsg = new QLabel(this);
	statWordCnt->setText("          ");
	statusOutput->insertWidget(0, statMsg, 1);
	statusOutput->insertPermanentWidget(1, statWordCnt, 0);
	playButton->setEnabled(false);
	solveButton->setEnabled(false);
	hintButton->setEnabled(false);
	finishButton->setEnabled(false);
	wordInput->setEnabled(false);

	puzzleLetrs[0] = puzzleLetr1;
	puzzleLetrs[1] = puzzleLetr2;
	puzzleLetrs[2] = puzzleLetr3;
	puzzleLetrs[3] = puzzleLetr4;
	puzzleLetrs[4] = puzzleLetr5;
	puzzleLetrs[5] = puzzleLetr6;
	puzzleLetrs[6] = puzzleLetr7;
	puzzleBox->setStyleSheet("QLineEdit#puzzleLetr7 {border: black; color: white; selection-background-color: grey; background-color: black;}");

	settingsDialog = new QDialog(this);
	settingsUi.setupUi(settingsDialog);
	settingsUi.spinBox_max->setRange(MMIN_SOL + MGAP_SOL, MMAX_SOL);
	settingsUi.spinBox_min->setRange(MMIN_SOL, MMAX_SOL - MGAP_SOL);

	connect(action_Exit, SIGNAL(triggered()), 
			this, SLOT(close()));
	connect(action_Instructions, SIGNAL(triggered()), 
			this, SLOT(showInst()));
	connect(action_About, SIGNAL(triggered()), 
			this, SLOT(showAbout()));
	connect(action_Settings, SIGNAL(triggered()),
			this, SLOT(showSettings()));

	connect(exitButton, SIGNAL(clicked()), 
			this, SLOT(close()));
	connect(playButton, SIGNAL(clicked()), 
			this, SLOT(playPressed()));
	connect(hintButton, SIGNAL(clicked()), 
			wordInput, SLOT(setFocus()));
	connect(hintButton, SIGNAL(clicked()), 
			spth, SLOT(hint()));
	connect(finishButton, SIGNAL(clicked()), 
			this, SLOT(finishPressed()));
	connect(solveButton, SIGNAL(clicked()), 
			this, SLOT(solvePressed()));
	connect(wordInput, SIGNAL(returnPressed()), 
			this, SLOT(wordEntered()));
	connect(wordInput, SIGNAL(textEdited(QString)), 
			this, SLOT(wordip(QString)));
	connect(wordsOutput, SIGNAL(selectionChanged()),
			wordsOutput, SLOT(copy()));

	connect(spth, SIGNAL(printWords(QString)), 
			wordsOutput, SLOT(setText(QString)));
	connect(spth, SIGNAL(printStatus(QString)), 
			statMsg, SLOT(setText(QString)));
	connect(spth, SIGNAL(printStatusTemp(QString, int)), 
			statusOutput, SLOT(showMessage(QString, int)));
	connect(spth, SIGNAL(printMsg(QString, QString)), 
			this, SLOT(printMsg(QString, QString)));
	connect(spth, SIGNAL(printWordCnt(QString)),
			statWordCnt, SLOT(setText(QString)));
	connect(spth, SIGNAL(fatalMsg(QString)), 
			this, SLOT(fatalMsg(QString)));
	connect(spth, SIGNAL(disableHints()), 
			this, SLOT(disableHints()));
	connect(spth, SIGNAL(printPuzzle(QString)), 
			this, SLOT(printPuzzle(QString)));
	connect(spth, SIGNAL(finished()), 
			this, SLOT(finishPressed()));
	connect(spth, SIGNAL(noDict(QString)), 
			this, SLOT(noDict(QString)));
	connect(spth, SIGNAL(spellathonReady()),
			this, SLOT(spellathonReady()));
	connect(spth, SIGNAL(wordchkimg(bool)),
			wordInput, SLOT(chk(bool)));

	connect(this, SIGNAL(play()), 
			spth, SLOT(play()));
	connect(this, SIGNAL(play_precomp()), 
			spth, SLOT(play_precomp()));
	connect(this, SIGNAL(finish()), 
			spth, SLOT(finish()));
	connect(this, SIGNAL(setWord(QString)), 
			spth, SLOT(setWord(QString)));
	connect(this, SIGNAL(solve(QString)), 
			spth, SLOT(solve(QString)));
	connect(this, SIGNAL(setSolRange(int, int)), 
			spth, SLOT(setSolRange(int, int)));
	connect(this, SIGNAL(setDoScore(bool)),
			spth, SLOT(setDoScore(bool)));

	connect(settingsUi.dictSelectButton, SIGNAL(clicked()),
			this, SLOT(changeDict()));
	connect(settingsDialog, SIGNAL(accepted()),
			this, SLOT(settingAccepted()));
	connect(settingsUi.spinBox_max, SIGNAL(valueChanged(int)),
			this, SLOT(adjustSbMin(int)));
	connect(settingsUi.spinBox_min, SIGNAL(valueChanged(int)),
			this, SLOT(adjustSbMax(int)));

	for (int i = 0; i < 7; i++)
		connect(puzzleLetrs[i], SIGNAL(textEdited(QString)), 
			SLOT(puzzleToUpper()));


	pendingDictChange = "";
	precomp = settings->value("dict/precomp", PRECOMP).toBool();
	QDir dp;
	resize(settings->value("mainwidget/size", size()).toSize());
	move(settings->value("mainwidget/pos", pos()).toPoint());

	this->show();
	spth->init(settings->value("dict/path", dp.absoluteFilePath(DICTPATH))
					.toString(), precomp);

	emit setSolRange(settings->value("solutions/min", MIN_SOL).toInt(), 
			 settings->value("solutions/max", MAX_SOL).toInt());
	emit setDoScore(settings->value("score/show", true).toBool());
}

void
MainWidget::wordip(QString text)
{
	if (wordip_state_init) {
		wordInput->clear();
		wordsBox->setStyleSheet("QLineEdit#wordInput {color: black; font: normal}");
		wordInput->setText(QString(text[0]));
		wordip_state_init = false;
	}
}

void 
MainWidget::adjustSbMin(int v)
{
	if (v - settingsUi.spinBox_min->value() >= MGAP_SOL)
		settingsUi.spinBox_min->setMaximum(v - MGAP_SOL);
}

void 
MainWidget::adjustSbMax(int v)
{
	if (settingsUi.spinBox_max->value() - v >= MGAP_SOL)
		settingsUi.spinBox_max->setMinimum(v + MGAP_SOL);
}

void 
MainWidget::showAbout()
{
	QMessageBox::information(this, "About", "Spellathon "VERSION
				 "\n\nCopyright (c) 2008 Girish Rao" 
				 "\n\nhttps://sourceforge.net/projects/spellathon");
}

void 
MainWidget::showSettings()
{
	settingsUi.spinBox_min->setValue(spth->minSol());
	settingsUi.spinBox_max->setValue(spth->maxSol());
	settingsUi.dictPathEdit->setText(spth->dict());
	settingsUi.precompCheckBox->setChecked(spth->isPrecomp());
	settingsUi.scoreCheckBox->setChecked(spth->isDoScore());
	settingsDialog->resize(settings->value("settingswidget/size", 
			       settingsDialog->size()).toSize());
	settingsDialog->exec();
}

void
MainWidget::changeDict()
{
	QString dpath = QFileDialog::getOpenFileName(settingsDialog, "Choose Dictionary", settings->value("dict/path").toString());

	if (dpath != "")
		settingsUi.dictPathEdit->setText(dpath);
}

void
MainWidget::settingAccepted()
{
	emit setSolRange(settingsUi.spinBox_min->value(), 
			 settingsUi.spinBox_max->value());

	QString d =  settingsUi.dictPathEdit->text();	
	QDir dp;
	bool pc = settingsUi.precompCheckBox->isChecked();

	settings->setValue("solutions/min", 
			   settingsUi.spinBox_min->value());
	settings->setValue("solutions/max", 
			   settingsUi.spinBox_max->value());

	settings->setValue("score/show",
			   settingsUi.scoreCheckBox->isChecked());
	emit setDoScore(settingsUi.scoreCheckBox->isChecked());

	settings->setValue("dict/path", d);
	settings->setValue("dict/precomp", pc);

	if (d != spth->dict() || pc != precomp) {
		if (spth->isIngame())
			pendingDictChange = d;
		else
			spth->init(d, pc);
	}

	precomp = pc;
	settings->setValue("settingswidget/size", settingsDialog->size());
}

void 
MainWidget::showInst()
{
	QDialog *d = new QDialog(this);
	QTextEdit *t = new QTextEdit(d);
	QPushButton *p = new QPushButton(d);
	QVBoxLayout *l = new QVBoxLayout(d);

	connect(p, SIGNAL(clicked()), d, SLOT(close()));
	t->setLineWrapMode(QTextEdit::NoWrap);
	d->move(this->pos().rx() + this->frameSize().width(), this->pos().ry());
#ifdef WIN32
	d->resize(350, 480);
#else
	d->resize(380, 500);
#endif /* WIN32 */
	d->setWindowTitle("Instructions");
	t->setReadOnly(true);
	l->addWidget(t);
	l->addWidget(p, 0, Qt::AlignCenter);
	p->setText("&Ok");
	t->setText("\nGame instructions\n"
		   "---------------------------------\n\n"
		   "o  Find all the words of four or more letters"
		   " from\n   the letters shown in the puzzle.\n\n"
		   "o  In making a word, each letter may be used "
		   "once only.\n\no  Each word must contain the "
		   "central letter.\n\no  There should be atleast"
		   " one seven letter word.\n\n\n"
		   "Notes\n"
		   "-------------\n\n"
		   "-  Hints will decrease your score.\n\n"
		   "-  To get the top score/rating you will "
		   "also need to be fast.\n\n"
		   "-  Ratings: Newbie, Average, Good, "
		   "Very good,\n   Outstanding, Inhuman.\n\n"
		   "-  To solve a puzzle enter the letters of puzzle "
		   "in the\n     corresponding input boxes and press Solve.\n\n"
		   );
	d->show();
}
	
void 
MainWidget::disableHints()
{
	hintButton->setEnabled(false);
	statusOutput->showMessage("No more hints.", 3000);
}

void
MainWidget::noDict(QString err)
{
	playButton->setEnabled(false);
	hintButton->setEnabled(false);
	solveButton->setEnabled(false);
	finishButton->setEnabled(false);
	for (int i = 0; i < 7; i++) 
		puzzleLetrs[i]->clear();
	wordInput->clear();
	wordsOutput->clear();
	puzzleLetr7->setFocus();
	QMessageBox::critical(this, "Dictionary", err);
}

void
MainWidget::spellathonReady()
{
	playButton->setEnabled(true);
	solveButton->setEnabled(true);
	playButton->setFocus();
}

void 
MainWidget::playPressed()
{
	if (spth->isInprogress()) {
		int r = QMessageBox::warning(this, "New game", 
					     	"Game in progress. Quit current game?", 
					     	QMessageBox::Cancel | 
					     	QMessageBox::Ok, 
						QMessageBox::Cancel);

		if (r == QMessageBox::Cancel)
			return;
	}

	if (pendingDictChange != "") {
		spth->init(pendingDictChange, precomp);
		pendingDictChange = "";
		if (!spth->isDictOpen())
			return;
	}

	for (int i = 0; i < 7; i++) 
		puzzleLetrs[i]->setText("");

	wordsOutput->setText("");
	wordInput->setEnabled(false);
	wordInput->clear();
	statMsg->clear();
	statWordCnt->setText("          ");
	playButton->setEnabled(false);
	hintButton->setEnabled(false);
	finishButton->setEnabled(false);
	solveButton->setEnabled(false);
	action_Settings->setEnabled(false);
	
	if (precomp)
		emit play_precomp();
	else
		emit play();

	action_Settings->setEnabled(true);
	statMsg->clear();
	playButton->setEnabled(true);

	if (spth->isIngame()) {
		hintButton->setEnabled(true);
		finishButton->setEnabled(true);
		wordInput->setEnabled(true);
		wordInput->setFocus();
		wordsBox->setStyleSheet("QLineEdit#wordInput { color: grey; font: italic}");
		wordInput->setText("Enter a word");
		wordInput->setCursorPosition(0);
		wordip_state_init = true;
	} else {
		for (int i = 0; i < 7; i++) 
			puzzleLetrs[i]->setReadOnly(false);

		solveButton->setEnabled(true);
	}
}

void 
MainWidget::fatalMsg(QString s)
{
	QMessageBox::critical(this, "Fatal Error", s);
	this->close();
}
	
void 
MainWidget::printMsg(QString t, QString s)
{
	QMessageBox::information(this, t, s);
}

void 
MainWidget::finishPressed()
{
	playButton->setEnabled(true);
	hintButton->setEnabled(false);
	finishButton->setEnabled(false);
	solveButton->setEnabled(true);
	playButton->setFocus();
	wordInput->clear();
	wordInput->setEnabled(false);

	for (int i = 0; i < 7; i++) 
		puzzleLetrs[i]->setReadOnly(false);

	emit finish();
}

void
MainWidget::solvePressed()
{
	wordsOutput->setText("");
	wordInput->setEnabled(false);
	statWordCnt->setText("          ");
	statMsg->clear();

	if (pendingDictChange != "") {
		spth->init(pendingDictChange, precomp);
		pendingDictChange = "";
	}

	for (int i = 0; i < 7; i++) {
		if (!puzzleLetrs[i]->isModified())
			goto nol;

		puzzle[i] = (puzzleLetrs[i]->text())[0].toLower();
		if (!puzzle[i].isLower()) 
			goto nol;
	}

	emit solve(puzzle);
	puzzleLetr7->setFocus();
	return;

nol:
	for (int i = 0; i < 7; i++) 
		puzzleLetrs[i]->clear();

	puzzleLetr7->setFocus();

	QMessageBox::information(this, "Solve puzzle", 
				 "Please enter the seven letters of the puzzle in the"
				 " corresponding input boxes and then press Solve.");

}

void 
MainWidget::wordEntered()
{
	emit setWord(wordInput->text());
	wordInput->clear();
}

void 
MainWidget::puzzleToUpper()
{
	for (int i = 0; i < 7; i++) {
		puzzleLetrs[i]->setText((puzzleLetrs[i]->text()).toUpper());
		puzzleLetrs[i]->setModified(true);
	}
}

void 
MainWidget::printPuzzle(QString s)
{
	QString::iterator it(s.begin()); 

	while(it != s.end()) { 
		*it = (*it).toUpper(); 
		it++;
	}

	for (int i = 0; i < 7; i++) {
		puzzleLetrs[i]->setText(QString(s[i]));
		puzzleLetrs[i]->setReadOnly(true);
	}

}

void
MainWidget::closeEvent(QCloseEvent *event)
{
	settings->setValue("mainwidget/size", size());
	settings->setValue("mainwidget/pos", pos());
	settings->sync();
	event->accept();
}

MainWidget::~MainWidget()
{
	delete spth;
	delete settings; 
}

