/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef SP_H
#define SP_H

#include "ui_main_widget.h"
#include "ui_settings_widget.h"
#include "spellathon_cls.h"

using Spellathon::SpellathonCls;

class QMainWindow;
class QLabel;
class QSettings;
class QCloseEvent;

class MainWidget : public QMainWindow , public Ui::MainWindow
{
	Q_OBJECT

public:
	MainWidget(QWidget *parent = 0);
	~MainWidget();
private:
	void closeEvent(QCloseEvent *event);
public slots:
	void printPuzzle(QString);
	void fatalMsg	(QString);
	void printMsg(QString, QString);
	void disableHints();
	void noDict(QString);
	void spellathonReady();
private slots:
	void playPressed();
	void solvePressed();
	void finishPressed();
	void wordEntered();
	void wordip(QString);
	void showInst();
	void showAbout();
	void showSettings();
	void puzzleToUpper();
	void adjustSbMin(int);
	void adjustSbMax(int);
	void settingAccepted();
	void changeDict();
signals:
	void setWord(QString w);
	void play();
	void play_precomp();
	void finish();
	void solve(QString s);
	void setSolRange(int, int);
	void setDoScore(bool);
private:
	SpellathonCls	*spth;
	QString		puzzle;
	QString		pendingDictChange;
	QLabel		*statWordCnt;
	QLabel		*statMsg;
	QDialog		*settingsDialog;
	QLineEdit	*puzzleLetrs[7];
	QSettings	*settings;
	bool		precomp;
	bool		wordip_state_init;
	Ui::Settings	settingsUi;
};

#endif
