/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cstring>
#include <algorithm>
#include <cctype>
#include <cassert>

#include "dispwordsbuf.h"
#include "util.h"

#ifdef WIN32
# define random rand
# define srandom srand
#endif

namespace Spellathon
{
	
	void 
	DispWordsBuf::init(const vector<string> &w)
	{
		words.clear();
		setwords.clear();
		hlevel = 0;
		words.reserve(512);
		setwords.reserve(512);

		assert(w.size() > 0);
		words = w;
		sz = (words.size() * fw) + 1;
	
		free(sol);
		free(hint);
		sol = (char *)memalloc(sz * sizeof(char));
		hint = (char *)memalloc(sz * sizeof(char));
	
		/* fill the solution buf with all the sol. words with required formatting */
		int i;
		for (i = 0; i < (int)words.size(); i++) {
			sprintf(&sol[i*fw], "%-*s", fw, words[i].c_str());
			if (i && i % wpl == 0)
				sol[(i*fw)-1] = '\n';
		}
		sol[(i*fw)-1] = '\n';
		sol[(i*fw)] = '\0';
	
		/* hint buf initially has all the words blanked out */
		for (int i = 0; i < sz; i++) {
			if (islower(sol[i])) 
				hint[i] = '-';
			else 
				hint[i] = sol[i];
		}
	}
	
	int 
	DispWordsBuf::setWord(const string w)
	{
		p = std::find(setwords.begin(), setwords.end(), w);
		if (p != setwords.end())
			/* already set */
			return 0;
	
		p = std::find(words.begin(), words.end(), w);
		if (p == words.end())
			/* not found */
			return -1;
	
		/* found */
		int idx = p - words.begin();
		strncpy(&hint[idx*fw], w.c_str(), strlen(w.c_str()));
		setwords.push_back(w);
	
		return 1;
	}
	
	bool 
	DispWordsBuf::setHint()
	{
		if (hlevel >= 2)
			/* only 2 hints allowed */
			return false;
	
		srandom(time(0));
	
		for (int i = 0; i < (int)words.size(); i++) {
			int c = words[i].size();
			int j = c/3;	/* reveal 1/3'rd of the letters */
			do {
				int r = randnum(0, words[i].size());
				if (!islower(hint[(i*fw) + r])) {
					hint[(i*fw) + r] = sol[(i*fw) + r];
					j--;
				}
			} while (c-- && j);
		}
	
		hlevel++;
		return true;
	}
	
}
