/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef DISPWORDSBUF_H
#define DISPWORDSBUF_H

#include <string>
#include <vector>

using std::vector;
using std::string;

namespace Spellathon
{
	
	class DispWordsBuf
	{
	public:
		DispWordsBuf() : sol(0), hint(0) {}
		~DispWordsBuf() { free(sol); free(hint); }
		void init(const vector<string> &w);	
		int setWord(const string w);
		bool setHint();
		const char* hintBuf() const { return hint; }
		char* solBuf() const { return sol; }
		int hintLevel() const { return hlevel; }
		int nSetWords() const { return (int)setwords.size(); }
		int nWords() const { return (int)words.size(); }
	private:
		char	*sol;		
		char	*hint;
		int	sz;		/* buffer size */
		int	hlevel;		/* hint level */
	
		static const int wpl = 4;	/* words per line */
		static const int fw = 9;	/* field width */
	
		vector<string> words;		/* solution words */
		vector<string> setwords;	/*correct words entered by usr*/
		vector<string>::iterator p;
	};
	
}

#endif
